@@ -1,15 +1,20 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
+
+#if defined(HAVE_MPI)
+#include <mpi.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include <time.h>
 #include <sys/stat.h>
-#include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GmshSocket.h"
 #include "Gmsh.h"
@@ -23,9 +28,13 @@
 #if defined(HAVE_ONELAB)
 #include "onelab.h"
 #endif
+#if !defined(HAVE_ONELAB2)
+#include "gmshLocalNetworkClient.h"
+#endif
 
-#if defined(HAVE_MPI)
-#include <mpi.h>
+#if defined(HAVE_ONELAB2)
+#include "OnelabDatabase.h"
+#include "NetworkUtils.h"
 #endif
 
 #if defined(HAVE_PETSC)
@@ -55,12 +64,22 @@ std::string Msg::_firstError;
 GmshMessage *Msg::_callback = 0;
 std::string Msg::_commandLine;
 std::string Msg::_launchDate;
+std::map<std::string, double> Msg::_commandLineNumbers;
+std::map<std::string, std::string> Msg::_commandLineStrings;
+#if !defined(HAVE_ONELAB2)
 GmshClient *Msg::_client = 0;
+#endif
 std::string Msg::_execName;
-#if defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB2)
+OnelabDatabase *OnelabDatabase::_instance = NULL;
+OnelabDatabase *Msg::_onelabClient = 0;
+#elif defined(HAVE_ONELAB)
 onelab::client *Msg::_onelabClient = 0;
+#endif
+#if defined(HAVE_ONELAB)
 onelab::server *onelab::server::_server = 0;
 #endif
+std::string Msg::_gmshOnelabAction = "";
 
 #if defined(HAVE_NO_VSNPRINTF)
 static int vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
@@ -80,20 +99,22 @@ static int vsnprintf(char *str, size_t size, const char *fmt, va_list ap)
 
 static void addGmshPathToEnvironmentVar(const std::string &name)
 {
-  std::vector<std::string> split = SplitFileName(CTX::instance()->argv0);
-  std::string path;
-  char *tmp = getenv(name.c_str());
-  if(tmp){
-    path = tmp;
+  std::string gmshPath = SplitFileName(CTX::instance()->exeFileName)[0];
+  if(gmshPath.size()){
+    std::string path;
+    char *tmp = getenv(name.c_str());
+    if(tmp){
+      path = tmp;
 #if defined(WIN32)
-    path += ";" + split[0];
+      path += ";" + gmshPath;
 #else
-    path += ":" + split[0];
+      path += ":" + gmshPath;
 #endif
+    }
+    else
+      path = gmshPath;
+    SetEnvironmentVar(name.c_str(), path.c_str());
   }
-  else
-    path = split[0];
-  SetEnvironmentVar(name.c_str(), path.c_str());
 }
 
 void Msg::Init(int argc, char **argv)
@@ -132,14 +153,15 @@ void Msg::Init(int argc, char **argv)
     _commandLine += argv[i];
   }
 
-  if(argc && argv){
-    CTX::instance()->argv0 = std::string(argv[0]);
-    // add the directory where the binary is installed to the path where Python
-    // looks for modules, and to the path for executables (this allows us to
-    // find the onelab.py module or subclients automatically)
-    addGmshPathToEnvironmentVar("PYTHONPATH");
-    addGmshPathToEnvironmentVar("PATH");
-  }
+  CTX::instance()->exeFileName = GetExecutableFileName();
+  if(CTX::instance()->exeFileName.empty() && argc && argv)
+    CTX::instance()->exeFileName = argv[0];
+
+  // add the directory where the binary is installed to the path where Python
+  // looks for modules, and to the path for executables (this allows us to find
+  // the onelab.py module or subclients automatically)
+  addGmshPathToEnvironmentVar("PYTHONPATH");
+  addGmshPathToEnvironmentVar("PATH");
 
   InitializeOnelab("Gmsh");
 }
@@ -246,15 +268,18 @@ void Msg::Fatal(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Fatal", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Error(str);
+#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    FlGui::instance()->check();
+    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
     std::string tmp = std::string("@C1@.") + "Fatal   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
     if(_firstError.empty()) _firstError = str;
-    FlGui::instance()->setLastStatus(FL_RED);
+    FlGui::instance()->setLastStatus
+      (CTX::instance()->guiColorScheme ? FL_DARK_RED : FL_RED);
     FlGui::instance()->saveMessages
       ((CTX::instance()->homeDir + CTX::instance()->errorFileName).c_str());
     fl_alert("A fatal error has occurred which will force Gmsh to abort.\n"
@@ -292,15 +317,19 @@ void Msg::Error(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Error", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Error(str);
+#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    FlGui::instance()->check();
-    std::string tmp = std::string("@C1@.") + "Error   : " + str;
+    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+    std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B72@." : "@C1@.")
+      + "Error   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
     if(_firstError.empty()) _firstError = str;
-    FlGui::instance()->setLastStatus(FL_RED);
+    FlGui::instance()->setLastStatus
+      (CTX::instance()->guiColorScheme ? FL_DARK_RED : FL_RED);
   }
 #endif
 
@@ -321,7 +350,7 @@ void Msg::Warning(const char *fmt, ...)
 {
   _warningCount++;
 
-  if(_commRank || _verbosity < 2) return;
+  if(_verbosity < 2) return;
 
   char str[5000];
   va_list args;
@@ -330,12 +359,14 @@ void Msg::Warning(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Warning", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Warning(str);
-
+#endif
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    FlGui::instance()->check();
-    std::string tmp = std::string("@C5@.") + "Warning : " + str;
+    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+    std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B152@." : "@C5@.")
+      + "Warning : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
     if(_firstWarning.empty()) _firstWarning = str;
     FlGui::instance()->setLastStatus();
@@ -347,14 +378,17 @@ void Msg::Warning(const char *fmt, ...)
     if(!streamIsFile(stderr) && streamIsVT100(stderr)){
       c0 = "\33[35m"; c1 = "\33[0m";  // magenta
     }
-    fprintf(stderr, "%sWarning : %s%s\n", c0, str, c1);
+    if(_commSize > 1)
+      fprintf(stderr, "%sWarning : [rank %3d] %s%s\n", c0, _commRank, str, c1);
+    else
+      fprintf(stderr, "%sWarning : %s%s\n", c0, str, c1);
     fflush(stderr);
   }
 }
 
 void Msg::Info(const char *fmt, ...)
 {
-  if(_commRank || _verbosity < 4) return;
+  if(_verbosity < 4) return;
 
   char str[5000];
   va_list args;
@@ -363,7 +397,9 @@ void Msg::Info(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Info", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
+#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -371,7 +407,7 @@ void Msg::Info(const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      FlGui::instance()->check();
+      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
       std::string tmp = std::string("Info    : ") + str;
       FlGui::instance()->addMessage(tmp.c_str());
     }
@@ -379,7 +415,10 @@ void Msg::Info(const char *fmt, ...)
 #endif
 
   if(CTX::instance()->terminal){
-    fprintf(stdout, "Info    : %s\n", str);
+    if(_commSize > 1)
+      fprintf(stdout, "Info    : [rank %3d] %s\n", _commRank, str);
+    else
+      fprintf(stdout, "Info    : %s\n", str);
     fflush(stdout);
   }
 }
@@ -390,7 +429,7 @@ void Msg::RequestRender()
 
 void Msg::Direct(const char *fmt, ...)
 {
-  if(_commRank || _verbosity < 3) return;
+  if(_verbosity < 3) return;
 
   char str[5000];
   va_list args;
@@ -399,7 +438,9 @@ void Msg::Direct(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Direct", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
+#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -407,8 +448,9 @@ void Msg::Direct(const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      FlGui::instance()->check();
-      std::string tmp = std::string("@C4@.") + str;
+      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+      std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B136@." : "@C4@.")
+        + str;
       FlGui::instance()->addMessage(tmp.c_str());
     }
   }
@@ -419,14 +461,17 @@ void Msg::Direct(const char *fmt, ...)
     if(!streamIsFile(stdout) && streamIsVT100(stdout)){
       c0 = "\33[34m"; c1 = "\33[0m";  // blue
     }
-    fprintf(stdout, "%s%s%s\n", c0, str, c1);
+    if(_commSize > 1)
+      fprintf(stdout, "%s[rank %3d] %s%s\n", c0, _commRank, str, c1);
+    else
+      fprintf(stdout, "%s%s%s\n", c0, str, c1);
     fflush(stdout);
   }
 }
 
 void Msg::StatusBar(bool log, const char *fmt, ...)
 {
-  if(_commRank || _verbosity < 4) return;
+  if(_verbosity < 4) return;
 
   char str[5000];
   va_list args;
@@ -435,7 +480,9 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
   va_end(args);
 
   if(_callback && log) (*_callback)("Info", str);
+#if !defined(HAVE_ONELAB2)
   if(_client && log) _client->Info(str);
+#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -443,7 +490,7 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      if(log) FlGui::instance()->check();
+      if(log && FlGui::instance()->in_main_thread()) FlGui::instance()->check();
       if(!log || _verbosity > 4)
 	FlGui::instance()->setStatus(str);
       if(log){
@@ -455,7 +502,10 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
 #endif
 
   if(log && CTX::instance()->terminal){
-    fprintf(stdout, "Info    : %s\n", str);
+    if(_commSize > 1)
+      fprintf(stdout, "Info    : [rank %3d] %s\n", _commRank, str);
+    else
+      fprintf(stdout, "Info    : %s\n", str);
     fflush(stdout);
   }
 }
@@ -485,7 +535,9 @@ void Msg::Debug(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Debug", str);
+#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
+#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
@@ -518,11 +570,13 @@ void Msg::ProgressMeter(int n, int N, bool log, const char *fmt, ...)
     va_end(args);
     sprintf(str2, "%3d%%    : %s", _progressMeterCurrent, str);
 
+#if !defined(HAVE_ONELAB2)
     if(_client) _client->Progress(str2);
+#endif
 
 #if defined(HAVE_FLTK)
     if(FlGui::available() && _verbosity > 4){
-      FlGui::instance()->check();
+      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
       FlGui::instance()->setProgress(str, (n > N - 1) ? 0 : n, 0, N);
     }
 #endif
@@ -583,7 +637,9 @@ void Msg::PrintErrorCounter(const char *title)
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    std::string col = _errorCount ? "@C1@." : "@C5@.";
+    std::string col = _errorCount ?
+      std::string(CTX::instance()->guiColorScheme ? "@B72@." : "@C1@.") :
+      std::string(CTX::instance()->guiColorScheme ? "@B152@." : "@C5@.");
     FlGui::instance()->addMessage((col + prefix + line).c_str());
     FlGui::instance()->addMessage((col + prefix + title).c_str());
     FlGui::instance()->addMessage((col + prefix + warn).c_str());
@@ -688,7 +744,9 @@ int Msg::GetAnswer(const char *question, int defaultval, const char *zero,
 
 bool Msg::UseOnelab()
 {
-#if defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB2)
+  return true;
+#elif defined(HAVE_ONELAB)
   return _onelabClient ? true : false;
 #else
   return false;
@@ -700,14 +758,22 @@ void Msg::SetOnelabNumber(std::string name, double val, bool visible)
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::number> numbers;
+#if defined(HAVE_ONELAB2)
+    _onelabClient->get(numbers, name, "Gmsh");
+#else
     _onelabClient->get(numbers, name);
+#endif
     if(numbers.empty()){
       numbers.resize(1);
       numbers[0].setName(name);
     }
     numbers[0].setValue(val);
     numbers[0].setVisible(visible);
+#if defined(HAVE_ONELAB2)
+    _onelabClient->set(numbers[0], "Gmsh");
+#else
     _onelabClient->set(numbers[0]);
+#endif
   }
 #endif
 }
@@ -717,14 +783,22 @@ void Msg::SetOnelabString(std::string name, std::string val, bool visible)
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::string> strings;
+#if defined(HAVE_ONELAB2)
+    _onelabClient->get(strings, name, "Gmsh");
+#else
     _onelabClient->get(strings, name);
+#endif
     if(strings.empty()){
       strings.resize(1);
       strings[0].setName(name);
     }
     strings[0].setValue(val);
     strings[0].setVisible(visible);
+#if defined(HAVE_ONELAB2)
+    _onelabClient->set(strings[0], "Gmsh");
+#else
     _onelabClient->set(strings[0]);
+#endif
   }
 #endif
 }
@@ -737,8 +811,8 @@ public:
   void sendMergeFileRequest(const std::string &name)
   {
     if(name.find(".geo") != std::string::npos){
-      MergePostProcessingFile(name, CTX::instance()->solver.autoShowLastStep,
-			      CTX::instance()->solver.autoHideNewViews, true);
+      MergePostProcessingFile(name, CTX::instance()->solver.autoShowViews,
+                              CTX::instance()->solver.autoShowLastStep, true);
       GModel::current()->setFileName(name);
     }
     else if((name.find(".opt") != std::string::npos)){
@@ -748,8 +822,8 @@ public:
       MergeFile(name);
     }
     else
-      MergePostProcessingFile(name, CTX::instance()->solver.autoShowLastStep,
-			      CTX::instance()->solver.autoHideNewViews, true);
+      MergePostProcessingFile(name, CTX::instance()->solver.autoShowViews,
+                              CTX::instance()->solver.autoShowLastStep, true);
   }
   void sendInfo(const std::string &msg){ Msg::Info("%s", msg.c_str()); }
   void sendWarning(const std::string &msg){ Msg::Warning("%s", msg.c_str()); }
@@ -759,7 +833,46 @@ public:
 
 void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
 {
-#if defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB2)
+  if(sockname.empty()){
+    if(name != "Gmsh"){ // load db from file:
+      FILE *fp = Fopen(name.c_str(), "rb");
+      if(fp){
+        OnelabDatabase::instance()->fromFile(fp);
+        fclose(fp);
+      }
+      else
+        Error("Error loading onelab database '%s'", name.c_str());
+    }
+    _onelabClient = OnelabDatabase::instance();
+  }
+  else{
+    UInt32 address = 0;
+    UInt16 port = 1148;
+    size_t colon = sockname.find(':');
+    if(colon != std::string::npos) {
+      address = ip4_inet_pton(sockname.substr(0,colon).c_str());
+      port = atoi(sockname.substr(colon+1).c_str());
+    }
+    GmshNetworkClient *c = OnelabDatabase::instance()->useAsNetworkClient(address, port, name);
+    if(c == NULL) {
+      Error("Unable to connect ONELAB server (%s)", sockname.c_str());
+      Exit(1);
+    }
+    _onelabClient = OnelabDatabase::instance();
+
+    SetOnelabNumber(name + "/UseCommandLine", 1, false);
+    SetOnelabString(name + "/FileExtension", ".geo", false);
+    SetOnelabString(name + "/9CheckCommand", "-", false);
+    SetOnelabString(name + "/9ComputeCommand", "-3", false);
+    std::vector<onelab::string> ps;
+    _onelabClient->get(ps, name + "/Action", "Gmsh");
+    if(ps.size()){
+      Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
+      if(ps[0].getValue() == "initialize") Exit(0);
+    }
+  }
+#elif defined(HAVE_ONELAB)
   if(_onelabClient) delete _onelabClient;
   if(sockname.empty()){
     _onelabClient = new localGmsh();
@@ -786,7 +899,7 @@ void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name + "/Action");
     if(ps.size()){
-      //Info("Performing OneLab '%s'", ps[0].getValue().c_str());
+      //Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
       if(ps[0].getValue() == "initialize") Exit(0);
     }
   }
@@ -797,30 +910,29 @@ void Msg::LoadOnelabClient(const std::string &clientName, const std::string &soc
 {
 #if defined(HAVE_ONELAB)
   onelab::remoteNetworkClient *client = 0;
-  client = new onelab::remoteNetworkClient(clientName,sockName);
+  client = new onelab::remoteNetworkClient(clientName, sockName);
   if(client){
     std::string action, cmd;
     std::vector<onelab::string> ps;
     client->get(ps,clientName+"/Action");
     if(ps.size() && ps[0].getValue().size())
       action.assign(ps[0].getValue());
-
     //cmd.assign("");
     if(!action.compare("compute")){
       std::vector<onelab::string> ps;
       client->get(ps,clientName+"/FullCmdLine");
       if(ps.size() && ps[0].getValue().size())
-	cmd.assign(ps[0].getValue());
+        cmd.assign(ps[0].getValue());
 
       if(cmd.size()){
-	Msg::Info("Loader calls <%s>",cmd.c_str());
-	//client->sendInfo(strcat("Loader calls",cmd.c_str()));
-	std::cout << "Loader calls " << cmd << std::endl;
-	SystemCall(cmd.c_str(),true); //true->blocking
+        Msg::Info("Loader calls <%s>", cmd.c_str());
+        //client->sendInfo(strcat("Loader calls",cmd.c_str()));
+        std::cout << "Loader calls " << cmd << std::endl;
+        SystemCall(cmd.c_str(), true); //true->blocking
       }
       else
-	Msg::Info("No full command line found for <%s>",
-		    clientName.c_str());
+        Msg::Info("No full command line found for <%s>",
+                  clientName.c_str());
     }
     Msg::Info("Stopping client <%s>", clientName.c_str());
     delete client;
@@ -848,6 +960,7 @@ static void _setStandardOptions(onelab::parameter *p,
   // numbers
   if(fopt.count("Visible")) p->setVisible(fopt["Visible"][0] ? true : false);
   if(fopt.count("ReadOnly")) p->setReadOnly(fopt["ReadOnly"][0] ? true : false);
+  if(fopt.count("NeverChanged")) p->setNeverChanged(fopt["NeverChanged"][0] ? true : false);
   if(fopt.count("ReadOnlyRange"))
     p->setAttribute("ReadOnlyRange", fopt["ReadOnlyRange"][0] ? "1" : "0");
   if(fopt.count("AutoCheck"))
@@ -878,7 +991,12 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
                                   std::map<std::string, std::vector<std::string> > &copt)
 {
 #if defined(HAVE_ONELAB)
-  if(!_onelabClient || val.empty()) return;
+  if(!_onelabClient) return;
+
+  if(val.empty()){
+    Msg::Error("No value to exchange with ONELAB");
+    return;
+  }
 
   std::string name;
   if(copt.count("Name"))
@@ -887,13 +1005,17 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(name.empty()){
     if(copt.size() || fopt.size())
       Msg::Error("From now on you need to use the `Name' attribute to create a "
-                 "OneLab parameter: `Name \"%s\"'",
+                 "ONELAB parameter: `Name \"%s\"'",
                  _getParameterName(key, copt).c_str());
     return;
   }
 
   std::vector<onelab::number> ps;
+#if defined(HAVE_ONELAB2)
+  _onelabClient->get(ps, name, "Gmsh");
+#else
   _onelabClient->get(ps, name);
+#endif
   bool noRange = true, noChoices = true, noLoop = true;
   bool noGraph = true, noClosed = true;
   if(ps.size()){
@@ -973,8 +1095,12 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(noClosed && fopt.count("Closed"))
     ps[0].setAttribute("Closed", fopt["Closed"][0] ? "1" : "0");
   _setStandardOptions(&ps[0], fopt, copt);
+#if defined(HAVE_ONELAB2)
+  _onelabClient->set(ps[0], "Gmsh");
+#else
   _onelabClient->set(ps[0]);
 #endif
+#endif
 }
 
 void Msg::ExchangeOnelabParameter(const std::string &key,
@@ -983,7 +1109,7 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
                                   std::map<std::string, std::vector<std::string> > &copt)
 {
 #if defined(HAVE_ONELAB)
-  if(!_onelabClient || val.empty()) return;
+  if(!_onelabClient) return;
 
   std::string name;
   if(copt.count("Name"))
@@ -992,7 +1118,7 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(name.empty()){
     if(copt.size() || fopt.size())
       Msg::Error("From now on you need to use the `Name' attribute to create a "
-                 "OneLab parameter: `Name \"%s\"'",
+                 "ONELAB parameter: `Name \"%s\"'",
                  _getParameterName(key, copt).c_str());
     return;
   }
@@ -1027,8 +1153,12 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(noMultipleSelection && copt.count("MultipleSelection"))
     ps[0].setAttribute("MultipleSelection", copt["MultipleSelection"][0]);
   _setStandardOptions(&ps[0], fopt, copt);
+#if defined(HAVE_ONELAB2)
+  _onelabClient->set(ps[0], "Gmsh");
+#else
   _onelabClient->set(ps[0]);
 #endif
+#endif
 }
 
 void Msg::UndefineOnelabParameter(const std::string &name)
@@ -1069,9 +1199,23 @@ void Msg::ImportPhysicalsAsOnelabRegions()
 #endif
 }
 
+void Msg::RunOnelabClient(const std::string &name, const std::string &command)
+{
+#if defined(HAVE_ONELAB) && !defined(HAVE_ONELAB2)
+  gmshLocalNetworkClient client(name, command, "", true);
+  client.run();
+#endif
+}
+
 void Msg::FinalizeOnelab()
 {
-#if defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB2)
+  if(_onelabClient) {
+    _onelabClient->finalize();
+    delete _onelabClient;
+    _onelabClient = 0;
+  }
+#elif defined(HAVE_ONELAB)
   if(_onelabClient){
     delete _onelabClient;
     _onelabClient = 0;
