@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -11,9 +11,15 @@
 #include <string>
 #include <stdarg.h>
 
+#include "GmshConfig.h"
+
 class GmshClient;
 namespace onelab{ class client; }
 
+#if defined(HAVE_ONELAB2)
+class OnelabDatabase;
+#endif
+
 // the external message handler
 class GmshMessage{
  public:
@@ -41,10 +47,22 @@ class Msg {
   static GmshMessage *_callback;
   // command-line and startup time
   static std::string _commandLine, _launchDate;
+  // command-line-specified numbers and strings
+  static std::map<std::string, double> _commandLineNumbers;
+  static std::map<std::string, std::string> _commandLineStrings;
+#if !defined(HAVE_ONELAB2)
   // communication with Gmsh when run remotely
   static GmshClient *_client;
+#endif
+#if defined(HAVE_ONELAB2)
+  // communication with onelab server (replace _client and old _onelabClient)
+  static OnelabDatabase *_onelabClient;
+#elif defined(HAVE_ONELAB)
   // communication with onelab server
   static onelab::client *_onelabClient;
+#endif
+  // internal onelab status for Gmsh parser
+  static std::string _gmshOnelabAction;
   // executable name
   static std::string _execName;
  public:
@@ -65,6 +83,14 @@ class Msg {
   static int GetVerbosity(){ return _verbosity; }
   static std::string GetLaunchDate(){ return _launchDate; }
   static std::string GetCommandLineArgs(){ return _commandLine; }
+  static std::map<std::string, double> &GetCommandLineNumbers()
+  {
+    return _commandLineNumbers;
+  }
+  static std::map<std::string, std::string> &GetCommandLineStrings()
+  {
+    return _commandLineStrings;
+  }
   static void Fatal(const char *fmt, ...);
   static void Error(const char *fmt, ...);
   static void Warning(const char *fmt, ...);
@@ -94,12 +120,22 @@ class Msg {
   static void SetExecutableName(const std::string &name) { _execName.assign(name); }
   static std::string GetExecutableName() { return _execName; }
   static void LoadOnelabClient(const std::string &name, const std::string &sockName);
+#if !defined(HAVE_ONELAB2)
   static GmshClient *GetGmshClient(){ return _client; }
+#else
+  static int GetGmshClient(){ return 1; }
+#endif
+#if defined(HAVE_ONELAB2)
+  static OnelabDatabase *GetOnelabClient(){ return _onelabClient; }
+#elif defined(HAVE_ONELAB)
   static onelab::client *GetOnelabClient(){ return _onelabClient; }
+#endif
   static void FinalizeOnelab();
   static bool UseOnelab();
   static void SetOnelabNumber(std::string name, double val, bool visible);
   static void SetOnelabString(std::string name, std::string val, bool visible);
+  static void SetGmshOnelabAction(std::string action){ _gmshOnelabAction = action; }
+  static std::string GetGmshOnelabAction(){ return _gmshOnelabAction; }
   static void ExchangeOnelabParameter(const std::string &key,
                                       std::vector<double> &val,
                                       std::map<std::string, std::vector<double> > &fopt,
@@ -109,6 +145,7 @@ class Msg {
                                       std::map<std::string, std::vector<double> > &fopt,
                                       std::map<std::string, std::vector<std::string> > &copt);
   static void UndefineOnelabParameter(const std::string &name);
+  static void RunOnelabClient(const std::string &name, const std::string &exe="");
   static void ImportPhysicalsAsOnelabRegions();
 };
 
