@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -28,9 +28,7 @@
 #ifndef _GMSH_SOCKET_H_
 #define _GMSH_SOCKET_H_
 
-#if !defined(ONELAB_LOADER)
 #include "GmshConfig.h"
-#endif
 
 #include <string>
 #include <stdio.h>
@@ -85,6 +83,8 @@ class GmshSocket{
     GMSH_SPEED_TEST          = 30,
     GMSH_PARAMETER_CLEAR     = 31,
     GMSH_PARAMETER_UPDATE    = 32,
+    GMSH_OPEN_PROJECT        = 33,
+    GMSH_CLIENT_CHANGED      = 34,
     GMSH_OPTION_1            = 100,
     GMSH_OPTION_2            = 101,
     GMSH_OPTION_3            = 102,
@@ -193,6 +193,7 @@ class GmshSocket{
   void Error(const char *str){ SendString(GMSH_ERROR, str); }
   void Progress(const char *str){ SendString(GMSH_PROGRESS, str); }
   void MergeFile(const char *str){ SendString(GMSH_MERGE_FILE, str); }
+  void OpenProject(const char *str){ SendString(GMSH_OPEN_PROJECT, str); }
   void ParseString(const char *str){ SendString(GMSH_PARSE_STRING, str); }
   void SpeedTest(const char *str){ SendString(GMSH_SPEED_TEST, str); }
   void Option(int num, const char *str)
@@ -325,11 +326,11 @@ class GmshServer : public GmshSocket{
  public:
   GmshServer() : GmshSocket(), _portno(-1) {}
   virtual ~GmshServer(){}
-  virtual int NonBlockingSystemCall(const char *str) = 0;
+  virtual int NonBlockingSystemCall(const std::string &exe, const std::string &args) = 0;
   virtual int NonBlockingWait(double waitint, double timeout, int socket=-1) = 0;
-  // start the client by launching "command" (command is supposed to contain
+  // start the client by launching "exe args" (args is supposed to contain
   // '%s' where the socket name should appear)
-  int Start(const char *command, const char *sockname, double timeout)
+  int Start(const char *exe, const char *args, const char *sockname, double timeout)
   {
     if(!sockname) throw "Invalid (null) socket name";
     _sockname = sockname;
@@ -394,10 +395,10 @@ class GmshServer : public GmshSocket{
       }
     }
 
-    if(command && strlen(command)){
-      char cmd[1024];
-      sprintf(cmd, command, _sockname.c_str());
-      NonBlockingSystemCall(cmd); // starts the solver
+    if((exe && strlen(exe)) || (args && strlen(args))){
+      char s[1024];
+      sprintf(s, args, _sockname.c_str());
+      NonBlockingSystemCall(exe, s); // starts the solver
     }
     else{
       timeout = 0.; // no command launched: don't set a timeout
