@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -7,6 +7,7 @@
 #include <string.h>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
+#include "GmshIO.h"
 #include "Options.h"
 #include "Geo.h"
 #include "GModel.h"
@@ -41,7 +42,11 @@
 #if defined(HAVE_FLTK)
 #include <FL/fl_ask.H>
 #include "FlGui.h"
+#if defined(HAVE_ONELAB2)
+#include "onelab2Group.h"
+#else
 #include "onelabGroup.h"
+#endif
 #include "graphicWindow.h"
 #include "drawContext.h"
 #endif
@@ -85,6 +90,12 @@ static void FinishUpBoundingBox()
     double l = sqrt(SQU(range[0]) + SQU(range[2]));
     CTX::instance()->min[1] -= l; CTX::instance()->max[1] += l;
   }
+
+  CTX::instance()->lc = sqrt(SQU(CTX::instance()->max[0] - CTX::instance()->min[0]) +
+                             SQU(CTX::instance()->max[1] - CTX::instance()->min[1]) +
+                             SQU(CTX::instance()->max[2] - CTX::instance()->min[2]));
+  for(int i = 0; i < 3; i++)
+    CTX::instance()->cg[i] = 0.5 * (CTX::instance()->min[i] + CTX::instance()->max[i]);
 }
 
 void SetBoundingBox(double xmin, double xmax,
@@ -95,12 +106,6 @@ void SetBoundingBox(double xmin, double xmax,
   CTX::instance()->min[1] = ymin; CTX::instance()->max[1] = ymax;
   CTX::instance()->min[2] = zmin; CTX::instance()->max[2] = zmax;
   FinishUpBoundingBox();
-  CTX::instance()->lc = sqrt(SQU(CTX::instance()->max[0] - CTX::instance()->min[0]) +
-                             SQU(CTX::instance()->max[1] - CTX::instance()->min[1]) +
-                             SQU(CTX::instance()->max[2] - CTX::instance()->min[2]));
-  for(int i = 0; i < 3; i++)
-    CTX::instance()->cg[i] = 0.5 * (CTX::instance()->min[i] + CTX::instance()->max[i]);
-
 }
 
 void SetBoundingBox(bool aroundVisible)
@@ -126,12 +131,6 @@ void SetBoundingBox(bool aroundVisible)
   CTX::instance()->min[1] = bb.min().y(); CTX::instance()->max[1] = bb.max().y();
   CTX::instance()->min[2] = bb.min().z(); CTX::instance()->max[2] = bb.max().z();
   FinishUpBoundingBox();
-  CTX::instance()->lc = sqrt(SQU(CTX::instance()->max[0] - CTX::instance()->min[0]) +
-                             SQU(CTX::instance()->max[1] - CTX::instance()->min[1]) +
-                             SQU(CTX::instance()->max[2] - CTX::instance()->min[2]));
-  for(int i = 0; i < 3; i++)
-    CTX::instance()->cg[i] = 0.5 * (CTX::instance()->min[i] + CTX::instance()->max[i]);
-
 }
 
 // FIXME: this is necessary for now to have an approximate CTX::instance()->lc
@@ -173,7 +172,7 @@ static void ComputeMaxEntityNum()
              GModel::current()->getMaxElementaryNumber(3));
 }
 
-static std::vector<FILE*> openedFiles;
+static std::vector<gmshFILE> openedFiles;
 
 int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
 {
@@ -184,19 +183,19 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
 
   // add 'b' for pure Windows programs: opening in text mode messes up
   // fsetpos/fgetpos (used e.g. for user-defined functions)
-  FILE *fp;
-  if(!(fp = Fopen(fileName.c_str(), "rb"))){
+  gmshFILE fp;
+  if(!(fp = gmshopen(fileName.c_str(), "rb"))){
     if(warnIfMissing)
       Msg::Warning("Unable to open file '%s'", fileName.c_str());
     return 0;
   }
 
-#if defined(HAVE_FLTK) && defined(HAVE_POST)
-  int numViewsBefore = PView::list.size();
-#endif
+  //#if defined(HAVE_FLTK) && defined(HAVE_POST)
+  // int numViewsBefore = PView::list.size();
+  //#endif
 
   std::string old_yyname = gmsh_yyname;
-  FILE *old_yyin = gmsh_yyin;
+  gmshFILE old_yyin = gmsh_yyin;
   int old_yyerrorstate = gmsh_yyerrorstate;
   int old_yylineno = gmsh_yylineno;
   int old_yyviewindex = gmsh_yyviewindex;
@@ -207,7 +206,7 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
   gmsh_yylineno = 1;
   gmsh_yyviewindex = 0;
 
-  while(!feof(gmsh_yyin)){
+  while(!gmsheof(gmsh_yyin)){
     gmsh_yyparse();
     if(gmsh_yyerrorstate > 20){
       if(gmsh_yyerrorstate != 999) // 999 is a volontary exit
@@ -219,7 +218,7 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
 
   if(close){
     gmsh_yyflush();
-    fclose(gmsh_yyin);
+    gmshclose(gmsh_yyin);
   }
   else{
     openedFiles.push_back(gmsh_yyin);
@@ -232,23 +231,47 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
   gmsh_yyviewindex = old_yyviewindex;
 
 #if defined(HAVE_FLTK) && defined(HAVE_POST)
-  if(FlGui::available())
-    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
+  if(FlGui::available()){
+    // this is not enough if the script creates and deletes views
+    //FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
+    FlGui::instance()->updateViews(true, false);
+  }
 #endif
 
   return 1;
 #endif
 }
 
-void ParseString(const std::string &str)
+static bool doSystemUncompress(std::string fileName, std::string noExt)
+{
+  std::ostringstream sstream;
+  sstream << "File '"<< fileName << "' is in gzip format.\n\n"
+          << "Do you want to uncompress it?";
+  if(Msg::GetAnswer(sstream.str().c_str(), 0, "Cancel", "Uncompress")){
+    if(SystemCall(std::string("gunzip -c ") + fileName + " > " + noExt, true))
+      Msg::Warning("Potentially failed to uncompress `%s': check directory permissions",
+                   fileName.c_str());
+    GModel::current()->setFileName(noExt);
+    return true;
+  }
+  return false;
+}
+
+void ParseString(const std::string &str, bool inCurrentModelDir)
 {
   if(str.empty()) return;
-  std::string fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
+  std::string fileName;
+  if(inCurrentModelDir)
+    fileName = FixRelativePath(GModel::current()->getFileName(),
+                               CTX::instance()->tmpFileName);
+  else
+    fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
   FILE *fp = Fopen(fileName.c_str(), "w");
   if(fp){
     fprintf(fp, "%s\n", str.c_str());
     fclose(fp);
     GModel::readGEO(fileName);
+    UnlinkFile(fileName);
   }
 }
 
@@ -262,7 +285,8 @@ static int defineSolver(const std::string &name)
   return NUM_SOLVERS - 1;
 }
 
-int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTitle)
+int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTitle,
+              bool setBoundingBox)
 {
   if(GModel::current()->getName() == ""){
     GModel::current()->setFileName(fileName);
@@ -276,7 +300,7 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
 
   // added 'b' for pure Windows programs, since some of these files
   // contain binary data
-  FILE *fp = Fopen(fileName.c_str(), "rb");
+  gmshFILE fp = gmshopen(fileName.c_str(), "rb");
   if(!fp){
     if(warnIfMissing)
       Msg::Warning("Unable to open file '%s'", fileName.c_str());
@@ -284,8 +308,8 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   }
 
   char header[256];
-  if(!fgets(header, sizeof(header), fp)){ fclose(fp); return 0; }
-  fclose(fp);
+  if(!gmshgets(header, sizeof(header), fp)){ gmshclose(fp); return 0; }
+  gmshclose(fp);
 
   Msg::StatusBar(true, "Reading '%s'...", fileName.c_str());
 
@@ -293,18 +317,18 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   std::string noExt = split[0] + split[1], ext = split[2];
 
   if(ext == ".gz") {
-    // the real solution would be to rewrite all our I/O functions in
-    // terms of gzFile, but until then, this is better than nothing
-    std::ostringstream sstream;
-    sstream << "File '"<< fileName << "' is in gzip format.\n\n"
-            << "Do you want to uncompress it?";
-    if(Msg::GetAnswer(sstream.str().c_str(), 0, "Cancel", "Uncompress")){
-      if(SystemCall(std::string("gunzip -c ") + fileName + " > " + noExt, true))
-        Msg::Error("Failed to uncompress `%s': check directory permissions",
-                   fileName.c_str());
-      GModel::current()->setFileName(noExt);
-      return MergeFile(noExt, false, setWindowTitle);
+#if defined(HAVE_COMPRESSED_IO) && defined(HAVE_LIBZ)
+    std::vector<std::string> subsplit = SplitFileName(noExt);
+    ext = subsplit[2];
+    if(ext != ".geo" && ext != ".GEO" &&
+       ext != ".unv" && ext != ".UNV"){
+      if(doSystemUncompress(fileName, noExt))
+        return MergeFile(noExt, false, setWindowTitle);
     }
+#else
+    if(doSystemUncompress(fileName, noExt))
+      return MergeFile(noExt, false, setWindowTitle);
+#endif
   }
 
   // force reading msh file even if wrong extension if the header
@@ -349,14 +373,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   else if(ext == ".diff" || ext == ".DIFF"){
     status = GModel::current()->readDIFF(fileName);
   }
-  else if(ext == ".pdf" || ext == ".PDF"){
-#if defined(HAVE_POPPLER)
-    status = gmshPopplerWrapper::instance()->load_from_file(fileName);
-#else
-    Msg::Error("Gmsh has to be compiled with POPPLER for displaying PDF documents");
-    status = 0;
-#endif
-  }
   else if(ext == ".med" || ext == ".MED" || ext == ".mmed" || ext == ".MMED" ||
           ext == ".rmed" || ext == ".RMED"){
     status = GModel::readMED(fileName);
@@ -423,21 +439,18 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
     GModel::current()->setName(split[1] + ".geo");
     GModel::current()->setFileName(split[0] + split[1] + ".geo");
     CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->solverToRun = num; // used in ONELAB2
     return 1;
   }
   else if(ext == ".py" || ext == ".PY" ||
+          ext == ".m" || ext == ".M" ||
           ext == ".exe" || ext == ".EXE"){
     int num = defineSolver(split[1]);
     opt_solver_executable(num, GMSH_SET, fileName);
     CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->solverToRun = num; // used in ONELAB2
     return 1;
   }
-#endif
-#if defined(HAVE_ONELAB_METAMODEL) && defined(HAVE_FLTK)
-  else if(ext == ".ol"){
-    // FIXME: this is a hack -- think about a better way
-    status = metamodel_cb(fileName);
-  }
 #endif
   else {
     CTX::instance()->geom.draw = 1;
@@ -460,10 +473,17 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
       if(status > 1) status = PView::readMSH(fileName);
 #endif
 #if defined(HAVE_MESH)
+      /*
+      This was introduced in r6039 by Koen to snap high order vertices on the
+      geometry. But it introduces subtle bugs when reading high-order
+      post-processing views ; we should have an explicit command to do this, and
+      not modify the mesh without warning just by reading a file --CG.
+
       if(CTX::instance()->mesh.order > 1)
         SetOrderN(GModel::current(), CTX::instance()->mesh.order,
                   CTX::instance()->mesh.secondOrderLinear,
                   CTX::instance()->mesh.secondOrderIncomplete);
+      */
 #endif
     }
 #if defined(HAVE_POST)
@@ -474,11 +494,14 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
 #endif
     else {
       status = GModel::readGEO(fileName);
+#ifdef HAVE_ONELAB2
+      Msg::SetOnelabString("Gmsh/1ModelName", fileName, false);
+#endif
     }
   }
 
   ComputeMaxEntityNum();
-  SetBoundingBox();
+  if(setBoundingBox) SetBoundingBox();
   CTX::instance()->geom.draw = 1;
   CTX::instance()->mesh.changed = ENT_ALL;
 
@@ -503,12 +526,11 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   // merge the associated option file if there is one
   if(!StatFile(fileName + ".opt"))
     MergeFile(fileName + ".opt");
-
   return status;
 }
 
-int MergePostProcessingFile(const std::string &fileName, bool showLastStep,
-                            bool hideNewViews, bool warnIfMissing)
+int MergePostProcessingFile(const std::string &fileName, int showViews,
+                            bool showLastStep, bool warnIfMissing)
 {
 #if defined(HAVE_POST)
   // check if there is a mesh in the file
@@ -550,22 +572,23 @@ int MergePostProcessingFile(const std::string &fileName, bool showLastStep,
     GModel *m = new GModel();
     GModel::setCurrent(m);
   }
-  int ret = MergeFile(fileName, warnIfMissing);
+  int ret = MergeFile(fileName, warnIfMissing, true,
+                      old->bounds().empty() ? true : false);
   GModel::setCurrent(old);
   old->setVisibility(1);
 
   // hide everything except the onelab X-Y graphs
-  if(hideNewViews){
+  if(showViews == 0){
     for(unsigned int i = 0; i < PView::list.size(); i++){
-      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "OneLab")
+      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "ONELAB")
         PView::list[i]->getOptions()->visible = 0;
     }
   }
-  else if(n < PView::list.size()){
+  else if(showViews == 2 && n < PView::list.size()){
     // if we created new views, assume we only want to see those (and the
     // onelab X-Y graphs)
     for(unsigned int i = 0; i < n; i++){
-      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "OneLab")
+      if(PView::list[i]->getData()->getFileName().substr(0, 6) != "ONELAB")
         PView::list[i]->getOptions()->visible = 0;
     }
   }
@@ -603,7 +626,7 @@ void ClearProject()
   // close the files that might have been left open by ParseFile
   if(openedFiles.size()){
     for(unsigned int i = 0; i < openedFiles.size(); i++)
-      fclose(openedFiles[i]);
+      gmshclose(openedFiles[i]);
     openedFiles.clear();
   }
   Msg::Info("Done clearing all models and views");
@@ -638,49 +661,51 @@ void OpenProject(const std::string &fileName, bool setWindowTitle)
     // reuse it
     GModel::current()->destroy();
     GModel::current()->getGEOInternals()->destroy();
-    // don't clear the parser variables if we just launched gmsh with the
-    // -string, -setstring or -setnumber command line options
-#if defined(HAVE_PARSER)
-    std::string c = Msg::GetCommandLineArgs();
-    if(c.find("-string") == std::string::npos &&
-       c.find("-setstring") == std::string::npos &&
-       c.find("-setnumber") == std::string::npos){
-      gmsh_yysymbols.clear();
-      gmsh_yystringsymbols.clear();
-    }
-#endif
   }
   else{
-    // if the current model is not empty make it invisible, clear the parser
-    // variables and add a new model
-#if defined(HAVE_PARSER)
-    gmsh_yysymbols.clear();
-    gmsh_yystringsymbols.clear();
-#endif
+    // if the current model is not empty make it invisible and add a new model
     new GModel();
     GModel::current(GModel::list.size() - 1);
   }
 
+  // clear parser variables, but keep -setnumber/-setstrings command line
+  // definitions
+#if defined(HAVE_PARSER)
+  gmsh_yysymbols.clear();
+  gmsh_yystringsymbols.clear();
+  std::map<std::string, double> cln(Msg::GetCommandLineNumbers());
+  for(std::map<std::string, double>::iterator it = cln.begin(); it != cln.end(); it++)
+    gmsh_yysymbols[it->first].value = std::vector<double>(1, it->second);
+  std::map<std::string, std::string> cls(Msg::GetCommandLineStrings());
+  for(std::map<std::string, std::string>::iterator it = cls.begin(); it != cls.end(); it++)
+    gmsh_yystringsymbols[it->first] = it->second;
+#endif
+
   // temporary hack until we fill the current GModel on the fly during parsing
   ResetTemporaryBoundingBox();
 
   // merge the file
-  if(MergeFile(fileName, false, setWindowTitle)) {
-    if(fileName != CTX::instance()->recentFiles.front())
-      CTX::instance()->recentFiles.insert
-        (CTX::instance()->recentFiles.begin(), fileName);
-    if(CTX::instance()->recentFiles.size() > 5)
-      CTX::instance()->recentFiles.resize(5);
-#if defined(HAVE_FLTK)
-    if(FlGui::available())
-      FlGui::instance()->graph[0]->fillRecentHistoryMenu();
-#endif
+  MergeFile(fileName, false, setWindowTitle);
+
+  // fill recent opened file list
+  std::vector<std::string> tmp = CTX::instance()->recentFiles;
+  CTX::instance()->recentFiles.clear();
+  CTX::instance()->recentFiles.push_back(fileName);
+  for(unsigned int i = 0; i < tmp.size(); i++){
+    if(tmp[i] != fileName)
+      CTX::instance()->recentFiles.push_back(tmp[i]);
   }
+  if(CTX::instance()->recentFiles.size() > 10)
+    CTX::instance()->recentFiles.resize(10);
+#if defined(HAVE_FLTK)
+  if(FlGui::available())
+    FlGui::instance()->graph[0]->fillRecentHistoryMenu();
+#endif
 
   // close the files that might have been left open by ParseFile
   if(openedFiles.size()){
     for(unsigned int i = 0; i < openedFiles.size(); i++)
-      fclose(openedFiles[i]);
+      gmshclose(openedFiles[i]);
     openedFiles.clear();
   }
 
