@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -20,7 +20,6 @@
 #include "Geo.h"
 #include "GmshDefines.h"
 
-
 GVertex *GeoFactory::addVertex(GModel *gm, double x, double y, double z, double lc)
 {
   int num =  gm->getMaxElementaryNumber(0) + 1;
@@ -164,7 +163,6 @@ GFace *GeoFactory::addPlanarFace(GModel *gm, std::vector< std::vector<GEdge *> >
 
 GRegion* GeoFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > faces)
 {
-
   //create surface loop
   int nLoops = faces.size();
   std::vector<SurfaceLoop *> vecLoops;
@@ -391,6 +389,9 @@ std::vector<GEntity*> GeoFactory::extrudeBoundaryLayer(GModel *gm, GEntity *e,
     }
   }
 
+  List_Delete(list_out);
+  List_Delete(tmp);
+
   return extrudedEntities;
 
   // //return the new created entity
@@ -424,6 +425,16 @@ std::vector<GEntity*> GeoFactory::extrudeBoundaryLayer(GModel *gm, GEntity *e,
 
 };
 
+void GeoFactory::healGeometry(GModel *gm, double tolerance)
+{
+  GModel *current = GModel::current();
+  GModel::setCurrent(gm);
+  ReplaceAllDuplicatesNew(tolerance);
+  gm->destroy();
+  gm->importGEOInternals();
+  GModel::setCurrent(current);
+}
+
 #if defined(HAVE_OCC)
 #include "OCCIncludes.h"
 #include "GModelIO_OCC.h"
@@ -601,7 +612,7 @@ GEdge *OCCFactory::addSpline(GModel *gm, const splineType &type,
   }
   else if (type == BSPLINE) {
 
-    Handle(Geom_BSplineCurve) Bez = GeomAPI_PointsToBSpline(ctrlPoints).Curve(); 
+    Handle(Geom_BSplineCurve) Bez = GeomAPI_PointsToBSpline(ctrlPoints).Curve();
 
     if (occv1 && occv2)
       occEdge = BRepBuilderAPI_MakeEdge(Bez,occv1->getShape(),occv2->getShape()).Edge();
@@ -1156,8 +1167,8 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
          && IsEqualG(BarycenterFace1.X()+FaceTranslationVector[0],BarycenterFace2.X())
          && IsEqualG(BarycenterFace1.Y()+FaceTranslationVector[1],BarycenterFace2.Y())
          && IsEqualG(BarycenterFace1.Z()+FaceTranslationVector[2],BarycenterFace2.Z())){
-        numFaceMaster = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace1);
-        numFaceSlave  = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace2);
+        numFaceMaster = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace1)->tag();
+        numFaceSlave  = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace2)->tag();
         //Msg::Info("Face %d (slave) is most likely Face %d (master) translated by (%.2e,%.2e,%.2e)!",
         //           numFaceSlave,numFaceMaster,FaceTranslationVector[0],FaceTranslationVector[1],
         //           FaceTranslationVector[2]);
@@ -1170,7 +1181,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
         for (TopExp_Explorer aEdgeExplorer1(aFace1,TopAbs_EDGE); aEdgeExplorer1.More();
              aEdgeExplorer1.Next()) {
           TopoDS_Edge aEdge1 = TopoDS::Edge(aEdgeExplorer1.Current());
-          int numEdgeMaster  = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge1);
+          int numEdgeMaster  = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge1)->tag();
           EdgeListMaster[i1] = numEdgeMaster;
           //i2=0;
           for (TopExp_Explorer aEdgeExplorer2(aFace2,TopAbs_EDGE); aEdgeExplorer2.More();
@@ -1188,7 +1199,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
                && IsEqualG(BarycenterEdge1.X()+FaceTranslationVector[0],BarycenterEdge2.X())
                && IsEqualG(BarycenterEdge1.Y()+FaceTranslationVector[1],BarycenterEdge2.Y())
                && IsEqualG(BarycenterEdge1.Z()+FaceTranslationVector[2],BarycenterEdge2.Z())){
-              int numEdgeSlave   = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge2);
+              int numEdgeSlave   = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge2)->tag();
               EdgeListSlave[i1]  = numEdgeSlave;
             }
           }
@@ -1240,136 +1251,6 @@ void OCCFactory::setPeriodicPairOfFaces(GModel *gm, int numFaceMaster, std::vect
   }
 }
 
-/* setPhysicalNumToEntitiesInBox allows to set a physical number to all entities
-   of a given type (0:vertex, 1:edge, 2:face, 3:volume) lying inside a 3D box
-   defined by 2 points */
-void OCCFactory::setPhysicalNumToEntitiesInBox(GModel *gm, int EntityType, int PhysicalGroupNumber,
-                                               std::vector<double> p1, std::vector<double> p2)
-{
-  std::vector<int> ListOfGVerticeTagsInbox;
-  std::vector<int> ListOfGEdgesTagsInbox;
-  std::vector<int> ListOfGFacesTagsInbox;
-  std::vector<int> ListOfGRegionsTagsInbox;
-
-  Standard_Real aXmin, aYmin, aZmin, aXmax, aYmax, aZmax;
-  TopoDS_Shape shape = gm->_occ_internals->getShape();
-  switch (EntityType) {
-  case 0 :
-    for(TopExp_Explorer aVertexExplorer(shape, TopAbs_VERTEX); aVertexExplorer.More(); aVertexExplorer.Next()){
-      TopoDS_Vertex aVertex = TopoDS::Vertex(aVertexExplorer.Current());
-      Bnd_Box VertexBB;
-      BRepBndLib::Add(aVertex,VertexBB);
-      VertexBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GVertexTag = gm->getOCCInternals()->getGTagOfOCCVertexByNativePtr(gm,aVertex);
-        //Msg::Info("This volume %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GVertexTag);
-        ListOfGVerticeTagsInbox.push_back(GVertexTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGVerticeTagsInbox.begin() ;
-         it != ListOfGVerticeTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GVertex *gv = gm->getVertexByTag(*it);
-      if(gv){
-        gv->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 1 :
-    for(TopExp_Explorer aEdgeExplorer(shape, TopAbs_EDGE); aEdgeExplorer.More(); aEdgeExplorer.Next()){
-      TopoDS_Edge aEdge = TopoDS::Edge(aEdgeExplorer.Current());
-      Bnd_Box EdgeBB;
-      BRepBndLib::Add(aEdge,EdgeBB);
-      EdgeBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GEdgeTag = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge);
-        //Msg::Info("This edge %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GEdgeTag);
-        ListOfGEdgesTagsInbox.push_back(GEdgeTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGEdgesTagsInbox.begin() ; it != ListOfGEdgesTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GEdge *ge = gm->getEdgeByTag(*it);
-      if(ge){
-        ge->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 2 :
-    for(TopExp_Explorer aFaceExplorer(shape, TopAbs_FACE); aFaceExplorer.More(); aFaceExplorer.Next()){
-      TopoDS_Face aFace = TopoDS::Face(aFaceExplorer.Current());
-      Bnd_Box FaceBB;
-      BRepBndLib::Add(aFace,FaceBB);
-      FaceBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GFaceTag = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace);
-        //Msg::Info("This face %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GFaceTag);
-        ListOfGFacesTagsInbox.push_back(GFaceTag);
-      }
-    }
-    Msg::Info("This faces have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGFacesTagsInbox.begin() ;
-         it != ListOfGFacesTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GFace *gf = gm->getFaceByTag(*it);
-      if(gf){
-        gf->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 3 :
-    for(TopExp_Explorer aSolidExplorer(shape, TopAbs_SOLID); aSolidExplorer.More();
-        aSolidExplorer.Next()){
-      TopoDS_Solid aSolid = TopoDS::Solid(aSolidExplorer.Current());
-      Bnd_Box SolidBB;
-      BRepBndLib::Add(aSolid,SolidBB);
-      SolidBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]
-         ){
-        int GSolidTag = gm->getOCCInternals()->getGTagOfOCCSolidByNativePtr(gm,aSolid);
-        //Msg::Info("This volume %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GSolidTag);
-        ListOfGRegionsTagsInbox.push_back(GSolidTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGRegionsTagsInbox.begin() ;
-         it != ListOfGRegionsTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GRegion *gr = gm->getRegionByTag(*it);
-      if(gr){
-        gr->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  default:
-    Msg::Error("First argument of setPhysicalNumToEntitiesInBox should be 0,1,2 or 3 (Vertice, Edge, Surface or Volume)");
-  }
-}
-
 void OCCFactory::fillet(GModel *gm, std::vector<int> edges, double radius)
 {
   try{
@@ -1568,6 +1449,18 @@ GEntity *OCCFactory::addPipe(GModel *gm, GEntity *base, std::vector<GEdge *> wir
   return ret;
 }
 
+void OCCFactory::healGeometry(GModel *gm, double tolerance)
+{
+  if (tolerance < 0.)
+    tolerance = CTX::instance()->geom.tolerance;
+  if (!gm || !gm->_occ_internals)
+    return;
+  //gm->_occ_internals->healGeometry(tolerance, false, false, false, true, false, false);
+  gm->_occ_internals->healGeometry(tolerance, true, true, true, true, true, true);
+  gm->_occ_internals->buildLists();
+  gm->_occ_internals->buildGModel(gm);
+}
+
 //Prepare SGEOM integration
 #if defined(HAVE_SGEOM) && defined(HAVE_OCC)
 
@@ -1607,6 +1500,12 @@ GRegion* SGEOMFactory::addVolume(GModel *gm, std::vector<std::vector<GFace *> >
   return 0;
 }
 
+void SGEOMFactory::healGeometry(GModel *gm, double tolerance)
+{
+  Msg::Error("healGeometry not implemented yet for SGEOMFactory");
+  return 0;
+}
+
 #endif
 
 #endif
