@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -22,6 +22,7 @@
 #include "GEntity.h"
 #include "StringUtils.h"
 #include "Numeric.h"
+#include "CondNumBasis.h"
 #include "Context.h"
 
 #define SQU(a)      ((a)*(a))
@@ -48,6 +49,89 @@ MElement::MElement(int num, int part) : _visible(1)
   }
 }
 
+MElement* MElement::createElement(int tag, const std::vector<MVertex*> &vertices,
+                                  int num, int part)
+{
+  const int type = ElementType::ParentTypeFromTag(tag);
+  const int order = ElementType::OrderFromTag(tag);
+  const bool serendipity = ElementType::SerendipityFromTag(tag) > 1;
+
+  if (order == 0) {
+    Msg::Error("p0 elements can not be created (tag %d)", tag);
+    return NULL;
+  }
+
+  switch (type) {
+
+  case TYPE_PNT:
+    return new MPoint(vertices, num, part);
+
+  case TYPE_LIN:
+    if (order == 1)
+      return new MLine(vertices, num, part);
+    else if (order == 2)
+      return new MLine3(vertices, num, part);
+    else
+      return new MLineN(vertices, num, part);
+
+  case TYPE_TRI:
+    if (order == 1)
+      return new MTriangle(vertices, num, part);
+    else if (order == 2)
+      return new MTriangle6(vertices, num, part);
+    else
+      return new MTriangleN(vertices, order, num, part);
+
+  case TYPE_QUA:
+    if (order == 1)
+      return new MQuadrangle(vertices, num, part);
+    else if (order == 2 && serendipity)
+      return new MQuadrangle8(vertices, num, part);
+    else if (order == 2)
+      return new MQuadrangle9(vertices, num, part);
+    else
+      return new MQuadrangleN(vertices, order, num, part);
+
+  case TYPE_TET:
+    if (order == 1)
+      return new MTetrahedron(vertices, num, part);
+    else if (order == 2)
+      return new MTetrahedron10(vertices, num, part);
+    else
+      return new MTetrahedronN(vertices, order, num, part);
+
+  case TYPE_PYR:
+    if (order == 1)
+      return new MPyramid(vertices, num, part);
+    else
+      return new MPyramidN(vertices, order, num, part);
+
+  case TYPE_PRI:
+    if (order == 1)
+      return new MPrism(vertices, num, part);
+    else if (order == 2 && serendipity)
+      return new MPrism15(vertices, num, part);
+    else if (order == 2)
+      return new MPrism18(vertices, num, part);
+    else
+      return new MPrismN(vertices, order, num, part);
+
+  case TYPE_HEX:
+    if (order == 1)
+      return new MHexahedron(vertices, num, part);
+    else if (order == 2 && serendipity)
+      return new MHexahedron20(vertices, num, part);
+    else if (order == 2)
+      return new MHexahedron27(vertices, num, part);
+    else
+      return new MHexahedronN(vertices, order, num, part);
+
+  default:
+    break;
+  }
+  return NULL;
+}
+
 void MElement::_getEdgeRep(MVertex *v0, MVertex *v1,
                            double *x, double *y, double *z, SVector3 *n,
                            int faceIndex)
@@ -112,7 +196,17 @@ double MElement::rhoShapeMeasure()
     return 0.;
 }
 
-double MElement::maxDistToStraight()
+double MElement::metricShapeMeasure()
+{
+  return MetricBasis::minRCorner(this);
+}
+
+double MElement::metricShapeMeasure2()
+{
+  return MetricBasis::boundMinR(this);
+}
+
+double MElement::maxDistToStraight() const
 {
   const nodalBasis *lagBasis = getFunctionSpace();
   const fullMatrix<double> &uvw = lagBasis->points;
@@ -135,7 +229,7 @@ double MElement::maxDistToStraight()
   return maxdx;
 }
 
-void MElement::scaledJacRange(double &jmin, double &jmax, GEntity *ge)
+void MElement::scaledJacRange(double &jmin, double &jmax, GEntity *ge) const
 {
   jmin = jmax = 1.0;
 #if defined(HAVE_MESH)
@@ -150,7 +244,7 @@ void MElement::scaledJacRange(double &jmin, double &jmax, GEntity *ge)
     SVector3 geoNorm(0.,0.,0.);
     // ... correct Jacobian sign with geometrical normal
     for (int i=0; i<jac->getNumPrimMapNodes(); i++) {
-      MVertex *vert = getVertex(i);
+      const MVertex *vert = getVertex(i);
       if (vert->onWhat() == ge) {
         double u, v;
         vert->getParameter(0,u);
@@ -175,6 +269,105 @@ void MElement::scaledJacRange(double &jmin, double &jmax, GEntity *ge)
 #endif
 }
 
+void MElement::idealJacRange(double &jmin, double &jmax, GEntity *ge)
+{
+  jmin = jmax = 1.0;
+#if defined(HAVE_MESH)
+  const JacobianBasis *jac = getJacobianFuncSpace();
+  const int numJacNodes = jac->getNumJacNodes();
+  fullMatrix<double> nodesXYZ(jac->getNumMapNodes(),3);
+  getNodesCoord(nodesXYZ);
+  fullVector<double> iJi(numJacNodes), Bi(numJacNodes);
+  jac->getSignedIdealJacobian(nodesXYZ,iJi);
+  const int nEd = getNumEdges(), dim = getDim();
+  double sumEdLength = 0.;
+  for(int iEd = 0; iEd < nEd; iEd++)
+    sumEdLength += getEdge(iEd).length();
+  const double invMeanEdLength = double(nEd)/sumEdLength;
+  if (sumEdLength == 0.) {
+    jmin = 0.; jmax = 0.;
+    return;
+  }
+  double scale = (dim == 1.) ? invMeanEdLength :
+                 (dim == 2.) ? invMeanEdLength*invMeanEdLength :
+                 invMeanEdLength*invMeanEdLength*invMeanEdLength;
+  if (ge && (ge->dim() == 2) && ge->haveParametrization()) {
+    // If parametrized surface entity provided...
+    SVector3 geoNorm(0.,0.,0.);
+    // ... correct Jacobian sign with geometrical normal
+    for (int i=0; i<jac->getNumPrimMapNodes(); i++) {
+      const MVertex *vert = getVertex(i);
+      if (vert->onWhat() == ge) {
+        double u, v;
+        vert->getParameter(0,u);
+        vert->getParameter(1,v);
+        geoNorm += ((GFace*)ge)->normal(SPoint2(u,v));
+      }
+    }
+    if (geoNorm.normSq() == 0.) {
+      // If no vertex on surface or average is zero, take normal at barycenter
+      SPoint2 param = ((GFace*)ge)->parFromPoint(barycenter(true),false);
+      geoNorm = ((GFace*)ge)->normal(param);
+    }
+    fullMatrix<double> elNorm(1,3);
+    jac->getPrimNormal2D(nodesXYZ, elNorm, true);
+    const double dp = geoNorm(0) * elNorm(0,0) + geoNorm(1) * elNorm(0,1) +
+      geoNorm(2) * elNorm(0,2);
+    if (dp < 0.) scale = -scale;
+  }
+  iJi.scale(scale);
+  jac->lag2Bez(iJi,Bi);
+  jmin = *std::min_element(Bi.getDataPtr(),Bi.getDataPtr()+Bi.size());
+  jmax = *std::max_element(Bi.getDataPtr(),Bi.getDataPtr()+Bi.size());
+#endif
+}
+
+void MElement::signedInvCondNumRange(double &iCNMin, double &iCNMax, GEntity *ge)
+{
+  iCNMin = iCNMax = 1.0;
+#if defined(HAVE_MESH)
+  const CondNumBasis *cnb = BasisFactory::getCondNumBasis(getTypeForMSH());
+  const int numCNNodes = cnb->getNumCondNumNodes();
+  fullMatrix<double> nodesXYZ(cnb->getNumMapNodes(), 3), normals;
+  getNodesCoord(nodesXYZ);
+  if (getDim() == 2.) {
+    SVector3 nVec = getFace(0).normal();
+    normals.resize(1, 3);
+    normals(0, 0) = nVec[0]; normals(0, 1) = nVec[1]; normals(0, 2) = nVec[2];
+  }
+  if (ge && (ge->dim() == 2) && ge->haveParametrization()) {
+    // If parametrized surface entity provided...
+    SVector3 geoNorm(0., 0., 0.);
+    // ... correct Jacobian sign with geometrical normal
+    for (int i=0; i<getNumPrimaryVertices(); i++) {
+      const MVertex *vert = getVertex(i);
+      if (vert->onWhat() == ge) {
+        double u, v;
+        vert->getParameter(0, u);
+        vert->getParameter(1, v);
+        geoNorm += ((GFace*)ge)->normal(SPoint2(u, v));
+      }
+    }
+    if (geoNorm.normSq() == 0.) {
+      // If no vertex on surface or average is zero, take normal at barycenter
+      SPoint2 param = ((GFace*)ge)->parFromPoint(barycenter(true), false);
+      geoNorm = ((GFace*)ge)->normal(param);
+    }
+    const double dp = geoNorm(0) * normals(0, 0) + geoNorm(1) * normals(0, 1)
+                    + geoNorm(2) * normals(0, 2);
+    if (dp < 0.) {
+       normals(0, 0) = -normals(0, 0);
+       normals(0, 1) = -normals(0, 1);
+       normals(0, 2) = -normals(0, 2);
+    }
+  }
+  fullVector<double> invCondNum(numCNNodes);
+  cnb->getSignedInvCondNum(nodesXYZ, normals, invCondNum);
+  iCNMin = *std::min_element(invCondNum.getDataPtr(), invCondNum.getDataPtr()+numCNNodes);
+  iCNMax = *std::max_element(invCondNum.getDataPtr(), invCondNum.getDataPtr()+numCNNodes);
+#endif
+}
+
 void MElement::getNode(int num, double &u, double &v, double &w) const
 {
   // only for MElements that don't have a lookup table for this
@@ -309,6 +502,13 @@ std::string MElement::getInfoString()
   return std::string(tmp);
 }
 
+const nodalBasis* MElement::getFunctionSpace(int order, bool serendip) const
+{
+  if (order == -1) return BasisFactory::getNodalBasis(getTypeForMSH());
+  int tag = ElementType::getTag(getType(), order, serendip);
+  return tag ? BasisFactory::getNodalBasis(tag) : NULL;
+}
+
 static double _computeDeterminantAndRegularize(const MElement *ele, double jac[3][3])
 {
   double dJ = 0;
@@ -392,8 +592,8 @@ double MElement::getJacobian(double u, double v, double w, double jac[3][3]) con
       jac[j][1] += ver->y() * gg[j];
       jac[j][2] += ver->z() * gg[j];
     }
-    //    printf("GSF (%d,%g %g) = %g %g \n",i,u,v,gg[0],gg[1]);
   }
+
   return _computeDeterminantAndRegularize(this, jac);
 }
 
@@ -432,7 +632,7 @@ const {
   return _computeDeterminantAndRegularize(this, jac);
 }
 
-double MElement::getPrimaryJacobian(double u, double v, double w, double jac[3][3])
+double MElement::getPrimaryJacobian(double u, double v, double w, double jac[3][3]) const
 {
   jac[0][0] = jac[0][1] = jac[0][2] = 0.;
   jac[1][0] = jac[1][1] = jac[1][2] = 0.;
@@ -453,7 +653,7 @@ double MElement::getPrimaryJacobian(double u, double v, double w, double jac[3][
   return _computeDeterminantAndRegularize(this, jac);
 }
 
-void MElement::getSignedJacobian(fullVector<double> &jacobian, int o)
+void MElement::getSignedJacobian(fullVector<double> &jacobian, int o) const
 {
   const int numNodes = getNumVertices();
   fullMatrix<double> nodesXYZ(numNodes,3);
@@ -461,17 +661,71 @@ void MElement::getSignedJacobian(fullVector<double> &jacobian, int o)
   getJacobianFuncSpace(o)->getSignedJacobian(nodesXYZ,jacobian);
 }
 
-void MElement::getNodesCoord(fullMatrix<double> &nodesXYZ)
+void MElement::getNodesCoord(fullMatrix<double> &nodesXYZ) const
 {
   const int numNodes = getNumShapeFunctions();
   for (int i = 0; i < numNodes; i++) {
-    MVertex *v = getShapeFunctionNode(i);
+    const MVertex *v = getShapeFunctionNode(i);
     nodesXYZ(i,0) = v->x();
     nodesXYZ(i,1) = v->y();
     nodesXYZ(i,2) = v->z();
   }
 }
 
+double MElement::getEigenvaluesMetric(double u, double v, double w, double values[3]) const
+{
+  double jac[3][3];
+  getJacobian(u, v, w, jac);
+  GradientBasis::mapFromIdealElement(getType(), jac);
+
+  switch (getDim()) {
+  case 1:
+    values[0] = 0;
+    values[1] = -1;
+    values[2] = -1;
+    for (int d = 0; d < 3; ++d)
+      values[0] += jac[d][0] * jac[d][0];
+    return 1;
+
+  case 2:
+  {
+    fullMatrix<double> metric(2, 2);
+    for (int i = 0; i < 2; ++i) {
+      for (int j = 0; j < 2; ++j) {
+        for (int d = 0; d < 3; ++d)
+          metric(i, j) += jac[d][i] * jac[d][j];
+      }
+    }
+    fullVector<double> valReal(values, 2), valImag(2);
+    fullMatrix<double> vecLeft(2, 2), vecRight(2, 2);
+    metric.eig(valReal, valImag, vecLeft, vecRight, true);
+    values[2] = -1;
+    return std::sqrt(valReal(0) / valReal(1));
+  }
+
+  case 3:
+  {
+    fullMatrix<double> metric(3, 3);
+    for (int i = 0; i < 3; ++i) {
+      for (int j = 0; j < 3; ++j) {
+        for (int d = 0; d < 3; ++d)
+          metric(i, j) += jac[d][i] * jac[d][j];
+      }
+    }
+
+    fullVector<double> valReal(values, 3), valImag(3);
+    fullMatrix<double> vecLeft(3, 3), vecRight(3, 3);
+    metric.eig(valReal, valImag, vecLeft, vecRight, true);
+
+    return std::sqrt(valReal(0) / valReal(2));
+  }
+
+  default:
+    Msg::Error("wrong dimension for getEigenvaluesMetric function");
+    return -1;
+  }
+}
+
 void MElement::pnt(double u, double v, double w, SPoint3 &p) const
 {
   double x = 0., y = 0., z = 0.;
@@ -875,7 +1129,7 @@ void MElement::writeMSH2(FILE *fp, double version, bool binary, int num,
 }
 
 void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
-                        bool printGamma, bool printEta, bool printRho,
+                        bool printSICN, bool printGamma, bool printRho,
                         bool printDisto, double scalingFactor, int elementary)
 {
   const char *str = getStringForPOS();
@@ -902,6 +1156,13 @@ void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
       fprintf(fp, "%d", getNum());
     }
   }
+  if(printSICN){
+    double sICNMin = minSICNShapeMeasure();
+    for(int i = 0; i < n; i++){
+      if(first) first = false; else fprintf(fp, ",");
+      fprintf(fp, "%g", sICNMin);
+    }
+  }
   if(printGamma){
     double gamma = gammaShapeMeasure();
     for(int i = 0; i < n; i++){
@@ -909,13 +1170,6 @@ void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
       fprintf(fp, "%g", gamma);
     }
   }
-  if(printEta){
-    double eta = etaShapeMeasure();
-    for(int i = 0; i < n; i++){
-      if(first) first = false; else fprintf(fp, ",");
-      fprintf(fp, "%g", eta);
-    }
-  }
   if(printRho){
     double rho = rhoShapeMeasure();
     for(int i = 0; i < n; i++){
@@ -1051,6 +1305,8 @@ void MElement::writeUNV(FILE *fp, int num, int elementary, int physical)
 void MElement::writeMESH(FILE *fp, int elementTagType, int elementary,
                          int physical)
 {
+  if(physical < 0) reverse();
+
   for(int i = 0; i < getNumVertices(); i++)
     if (getTypeForMSH() == MSH_TET_10 && i == 8)
       fprintf(fp, " %d", getVertex(9)->getIndex());
@@ -1059,18 +1315,24 @@ void MElement::writeMESH(FILE *fp, int elementTagType, int elementary,
     else
       fprintf(fp, " %d", getVertex(i)->getIndex());
   fprintf(fp, " %d\n", (elementTagType == 3) ? _partition :
-          (elementTagType == 2) ? physical : elementary);
+          (elementTagType == 2) ? abs(physical) : elementary);
+
+  if(physical < 0) reverse();
 }
 
 void MElement::writeIR3(FILE *fp, int elementTagType, int num, int elementary,
                         int physical)
 {
+  if(physical < 0) reverse();
+
   int numVert = getNumVertices();
   fprintf(fp, "%d %d %d", num, (elementTagType == 3) ? _partition :
-          (elementTagType == 2) ? physical : elementary, numVert);
+          (elementTagType == 2) ? abs(physical) : elementary, numVert);
   for(int i = 0; i < numVert; i++)
     fprintf(fp, " %d", getVertex(i)->getIndex());
   fprintf(fp, "\n");
+
+  if(physical < 0) reverse();
 }
 
 void MElement::writeBDF(FILE *fp, int format, int elementTagType, int elementary,
@@ -1083,8 +1345,10 @@ void MElement::writeBDF(FILE *fp, int format, int elementTagType, int elementary
   const char *cont[4] = {"E", "F", "G", "H"};
   int ncont = 0;
 
+  if(physical < 0) reverse();
+
   int tag =  (elementTagType == 3) ? _partition : (elementTagType == 2) ?
-    physical : elementary;
+    abs(physical) : elementary;
 
   if(format == 0){ // free field format
     fprintf(fp, "%s,%d,%d", str, _num, tag);
@@ -1112,23 +1376,29 @@ void MElement::writeBDF(FILE *fp, int format, int elementTagType, int elementary
       fprintf(fp, "%-8s%-8s%-8s", "0.", "0.", "0.");
     fprintf(fp, "\n");
   }
+
+  if(physical < 0) reverse();
 }
 
-void MElement::writeDIFF(FILE *fp, int num, bool binary, int physical_property)
+void MElement::writeDIFF(FILE *fp, int num, bool binary, int physical)
 {
   const char *str = getStringForDIFF();
   if(!str) return;
 
+  if(physical < 0) reverse();
+
   int n = getNumVertices();
   if(binary){
     // TODO
   }
   else{
-    fprintf(fp, "%d %s %d ", num, str, physical_property);
+    fprintf(fp, "%d %s %d ", num, str, abs(physical));
     for(int i = 0; i < n; i++)
       fprintf(fp, " %d", getVertexDIFF(i)->getIndex());
     fprintf(fp, "\n");
   }
+
+  if(physical < 0) reverse();
 }
 
 void MElement::writeINP(FILE *fp, int num)
@@ -1560,3 +1830,19 @@ MElement *MElementFactory::create(int num, int type, const std::vector<int> &dat
 
   return element;
 }
+
+double MElement::skewness() {
+  double minsk = 1.0;
+  for (int i=0;i<getNumFaces();i++){
+    MFace f = getFace(i);
+    if (f.getNumVertices() == 3){
+      //      MTriangle t (f.getVertex(0),f.getVertex(1),f.getVertex(2));
+      //      minsk = std::min(minsk, t.etaShapeMeasure ());
+    }
+    else if (f.getNumVertices() == 4){
+      MQuadrangle q (f.getVertex(0),f.getVertex(1),f.getVertex(2),f.getVertex(3));
+      minsk = std::min(minsk, q.etaShapeMeasure ());
+    }
+  }
+  return minsk;
+}
