@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -6,10 +6,10 @@
 #ifndef _FULL_MATRIX_H_
 #define _FULL_MATRIX_H_
 
-#include <math.h>
-#include <stdio.h>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
+#include <cmath>
+#include <cstdio>
 
 template <class scalar> class fullMatrix;
 
@@ -111,6 +111,13 @@ class fullVector
   */
   inline const scalar * getDataPtr() const { return _data; }
 
+  /**
+     @return Returns a  pointer to this fullVector data.@n
+     This pointer will point to the following memory segment:
+     [(*this)(0), (*this)(1), ..., (*this)(this->size() - 1)].
+  */
+  inline scalar * getDataPtr() { return _data; }
+
   /**
      @param i A vector index between 0 and size() - 1.
      @returns Returns the ith scalar of this fullVector.
@@ -143,6 +150,12 @@ class fullVector
     return *this;
   }
 
+  void copy(const fullVector<scalar> &v, int i0, int ni, int desti0)
+  {
+    for(int i = i0, desti = desti0; i < i0 + ni; i++, desti++)
+      (*this)(desti) = v(i);
+  }
+
   // set
   /**
      @param r A vector index between 0 and size() - 1;
@@ -444,6 +457,9 @@ class fullMatrix
     return (*this)(r, c);
   }
 
+  inline const scalar * getDataPtr() const { return _data; }
+  inline scalar * getDataPtr() { return _data; }
+
   // operations
   inline void set(int r, int c, scalar v){
     #ifdef _DEBUG
